/* /////////////////////////////////////////////////////////////////////////////
 * File:        mman.h (sys/mman.h)
 *
 * Purpose:     Declaration of the mmap() and munmap() API functions.
 *
 * Created      18th December 2003
 * Updated:     3rd January 2004
 *
 * Author:      Matthew Wilson, Synesis Software Pty Ltd.
 *
 * License:     (Licensed under the Synesis Software Standard Source License)
 *
 *              Copyright (C) 2002, Synesis Software Pty Ltd.
 *
 *              All rights reserved.
 *
 *              www:        http://www.synesis.com.au/
 *                          http://www.synesis.com.au/software/
 *
 *              email:      software@synesis.com.au
 *                          software@synesis-group.com
 *
 *              Redistribution and use in source and binary forms, with or
 *              without modification, are permitted provided that the following
 *              conditions are met:
 *
 *              (i) Redistributions of source code must retain the above
 *              copyright notice and contact information, this list of
 *              conditions and the following disclaimer.
 *
 *              (ii) Any derived versions of this software (howsoever modified)
 *              remain the sole property of Synesis Software.
 *
 *              (iii) Any derived versions of this software (howsoever modified)
 *              remain subject to all these conditions.
 *
 *              (iv) Neither the name of Synesis Software nor the names of any
 *              subdivisions, employees or agents of Synesis Software, nor the
 *              names of any other contributors to this software may be used to
 *              endorse or promote products derived from this software without
 *              specific prior written permission.
 *
 *              This source code is provided by Synesis Software "as is" and any
 *              warranties, whether expressed or implied, including, but not
 *              limited to, the implied warranties of merchantability and
 *              fitness for a particular purpose are disclaimed. In no event
 *              shall the Synesis Software be liable for any direct, indirect,
 *              incidental, special, exemplary, or consequential damages
 *              (including, but not limited to, procurement of substitute goods
 *              or services; loss of use, data, or profits; or business
 *              interruption) however caused and on any theory of liability,
 *              whether in contract, strict liability, or tort (including
 *              negligence or otherwise) arising in any way out of the use of
 *              this software, even if advised of the possibility of such
 *              damage.
 *
 * ////////////////////////////////////////////////////////////////////////// */


#ifndef _SYNSOFT_INCL_H_SYS_MMAP
#define _SYNSOFT_INCL_H_SYS_MMAP

#ifndef _SYNSOFT_DOCUMENTATION_SKIP_SECTION
# define _SYNSOFT_VER_H_SYS_MMAP_MAJOR      1
# define _SYNSOFT_VER_H_SYS_MMAP_MINOR      1
# define _SYNSOFT_VER_H_SYS_MMAP_REVISION   3
# define _SYNSOFT_VER_H_SYS_MMAP_EDIT       5
#endif /* !_SYNSOFT_DOCUMENTATION_SKIP_SECTION */

#include <sys/types.h>

/* ////////////////////////////////////////////////////////////////////////// */

#ifndef WIN32
# error This file is only currently defined for Win32 compilation units
#endif /* WIN32 */

/* /////////////////////////////////////////////////////////////////////////////
 * Constants and definitions
 */

#define PROT_READ       0x1             /*!< \brief page can be read */ 
#define PROT_WRITE      0x2             /*!< \brief page can be written */ 
#define PROT_EXEC       0x4             /*!< \brief page can be executed */ 
#define PROT_NONE       0x0             /*!< \brief page can not be accessed */ 

#define MAP_PRIVATE     0x02            /*!< \brief Changes are private */ 
#define	MAP_ANONYMOUS	0x20			/*!< \brief Ignore fd and offset parameters */
#define	MAP_FIXED		0x10            /*!< \brief Interpret addr exactly */ 

#define MAP_FAILED	((void*)-1)			/*!< \brief Returned from mmap() when the mapping fails */

/* /////////////////////////////////////////////////////////////////////////////
 * API functions
 */

#ifdef __cplusplus
extern "C"
{
#endif /* __cplusplus */

/** \brief Maps a file into memory, and returns a pointer to it
 *
 * \param addr Suggested address for the mapping
 * \param len The number of bytes in the mapped region. The combination of 
 * len and offset must not exceed the length of the mapped file
 * \param prot Either PROT_NONE, or a combination of the other PROT_* flags
 * \param flags One of MAP_PRIVATE, MAP_ANONYMOUS or MAP_FIXED.
 * \param fd The file descriptor of the file to map, or -1 to allocate an anonymous map
 * \param offset The offset within the file to start the mapped region
 *
 * \retval pointer to mapped region if successful
 * \retval MAP_FAILED if failed
 *
 * \note The current implementation supports the following three modes: 
 * 1. Read-only file; 2. Read-write file; 3. Anonymous Read-write block of 
 * system paging file.
 */

void *mmap(void *addr, size_t len, int prot, int flags, int fd, off_t offset);

/** \brief Deletes a mapped region
 *
 * The munmap() system call deletes the mappings for the specified address
 * range, and causes further references to addresses within the range to
 * generate invalid memory references.
 *
 * \param addr The base address of the mapped region to unmap
 * \param len The length of the mapped region. Ignore in the Win32 implementation
 *
 * \retval 0 if successful
 * \retval -1 if failed
 */

int munmap(void *addr, size_t len);

/** \brief Writes any dirty pages within the given range to disk
 *
 * \param addr The base address of the mapped region
 * \param len The length of the mapped region to flush to disk. Will be rounded up 
 * to next page boundary.
 * \param flags Ignored
 */
int msync(void *addr, size_t len, int flags);

#ifdef __cplusplus
} // extern "C"
#endif /* __cplusplus */

/* ////////////////////////////////////////////////////////////////////////// */

#endif /* _SYNSOFT_INCL_H_SYS_MMAP */

/* ////////////////////////////////////////////////////////////////////////// */
