/* /////////////////////////////////////////////////////////////////////////////
 * File:        mmap.c
 *
 * Purpose:     mmap(),  for the Win32 platform.
 *
 * Created      18th December 2003
 * Updated:     19th December 2003
 *
 * Author:      Matthew Wilson, Synesis Software Pty Ltd.
 *
 * License:     (Licensed under the Synesis Software Standard Source License)
 *
 *              Copyright (C) 2002, Synesis Software Pty Ltd.
 *
 *              All rights reserved.
 *
 *              www:        http://www.synesis.com.au/
 *                          http://www.synesis.com.au/software/
 *
 *              email:      software@synesis.com.au
 *                          software@synesis-group.com
 *
 *              Redistribution and use in source and binary forms, with or
 *              without modification, are permitted provided that the following
 *              conditions are met:
 *
 *              (i) Redistributions of source code must retain the above
 *              copyright notice and contact information, this list of
 *              conditions and the following disclaimer.
 *
 *              (ii) Any derived versions of this software (howsoever modified)
 *              remain the sole property of Synesis Software.
 *
 *              (iii) Any derived versions of this software (howsoever modified)
 *              remain subject to all these conditions.
 *
 *              (iv) Neither the name of Synesis Software nor the names of any
 *              subdivisions, employees or agents of Synesis Software, nor the
 *              names of any other contributors to this software may be used to
 *              endorse or promote products derived from this software without
 *              specific prior written permission.
 *
 *              This source code is provided by Synesis Software "as is" and any
 *              warranties, whether expressed or implied, including, but not
 *              limited to, the implied warranties of merchantability and
 *              fitness for a particular purpose are disclaimed. In no event
 *              shall the Synesis Software be liable for any direct, indirect,
 *              incidental, special, exemplary, or consequential damages
 *              (including, but not limited to, procurement of substitute goods
 *              or services; loss of use, data, or profits; or business
 *              interruption) however caused and on any theory of liability,
 *              whether in contract, strict liability, or tort (including
 *              negligence or otherwise) arising in any way out of the use of
 *              this software, even if advised of the possibility of such
 *              damage.
 *
 * ////////////////////////////////////////////////////////////////////////// */


#define _SYNSOFT_VER_C_MMAP_MAJOR       1
#define _SYNSOFT_VER_C_MMAP_MINOR       1
#define _SYNSOFT_VER_C_MMAP_REVISION    4
#define _SYNSOFT_VER_C_MMAP_EDIT        6

#include <windows.h>

#include <errno.h>
#include <sys/mman.h>

#if defined(__BORLANDC__)
__declspec(dllimport) long _cdecl _get_osfhandle(int __handle);
#elif defined(__DMC__) || \
      defined(__INTEL_COMPILER) || \
      defined(__MWERKS__) || \
      defined(_MSC_VER)
__declspec(dllimport) long __cdecl _get_osfhandle(int __handle);
#else
# error Compiler not discriminated
#endif /* compiler */

/* /////////////////////////////////////////////////////////////////////////////
 * API functions
 */

static int win32_flags_from_mmap_(int prot, int flags, DWORD *cfmFlags, DWORD *mvofFlags)
{
	*cfmFlags	=	0;
	*mvofFlags	=	0;

	if(PROT_NONE == prot)
	{
		*cfmFlags	=	PAGE_NOACCESS;
		*mvofFlags	=	0;
	}
	else
	{
		if(prot & PROT_WRITE)
		{
			if((flags & MAP_PRIVATE))
			{
				*mvofFlags |= FILE_MAP_COPY;
			}
			else
			{
				*mvofFlags |= FILE_MAP_WRITE;
			}
		}
		else
		{
			*mvofFlags |= FILE_MAP_READ;
		}

		if(*mvofFlags & FILE_MAP_COPY)
		{
			*cfmFlags = PAGE_WRITECOPY;
		}
		else if(*mvofFlags & FILE_MAP_WRITE)
		{
			*cfmFlags = PAGE_READWRITE;
		}
		else
		{
			*cfmFlags = PAGE_READONLY;
		}
	}

	if(flags & MAP_ANONYMOUS)
	{
#if 0
		*cfmFlags |= SEC_RESERVE;
#endif /* 0 */
	}

	return 0;
}

/** 
 *
 * \retval pointer to mapped region if successful
 * \retval MAP_FAILED if failed
 */

void *mmap(void *addr, size_t len, int prot, int flags, int fd, off_t offset)
{
	/* Sanity checks first */
	int		errno_		=	0;
	HANDLE	fh			=	(fd < 0) ? INVALID_HANDLE_VALUE : (HANDLE)_get_osfhandle(fd);

	if(	NULL == addr && 
		0 != (flags & MAP_FIXED))
	{
		errno_ = ENOMEM;
	}
	else if(MAP_ANONYMOUS == (flags & MAP_ANONYMOUS) &&
			-1 != fd)
	{
		errno_ = EINVAL;
	}
	else if(MAP_ANONYMOUS == (flags & MAP_ANONYMOUS) &&
			(	0 == len ||
				0 != offset))
	{
		errno_ = EINVAL;
	}
	else
	{
		if(MAP_ANONYMOUS != (flags & MAP_ANONYMOUS))
		{
			DWORD	fileSize	=	GetFileSize(fh, NULL);

			if(	0xFFFFFFFF == fileSize &&
				ERROR_SUCCESS != GetLastError())
			{
				errno_ = EBADF;
			}
			else if(len + offset > fileSize)
			{
				errno_ = EINVAL;
			}
		}
	}

	if(0 != errno_)
	{
		errno = errno_;
		return MAP_FAILED;
	}
	else
	{
		DWORD	cfmFlags;
		DWORD	mvofFlags;

		errno_	=	win32_flags_from_mmap_(prot, flags, &cfmFlags, &mvofFlags);

		if(0 != errno_)
		{
			return MAP_FAILED;
		}
		else
		{
			HANDLE	hMap	=	CreateFileMapping(fh, NULL, cfmFlags, 0, len, NULL);

			if(NULL == hMap)
			{
				DWORD	dwErr	=	GetLastError();

				if(dwErr == ERROR_ACCESS_DENIED)
				{
					errno = EACCES;
				}
				else if(dwErr == ERROR_INVALID_PARAMETER)
				{
					errno = EINVAL;
				}
				else if(dwErr == ERROR_FILE_INVALID)
				{
					errno = EBADF;
				}

				return MAP_FAILED;
			}
			else
			{
				void	*pvMap	=	MapViewOfFileEx(hMap, mvofFlags, 0, offset, len, addr);

				CloseHandle(hMap);

				return pvMap;
			}
		}
	}
}

/** The munmap() system call deletes the mappings for the specified address
 * range, and causes further references to addresses within the range to
 * generate invalid memory references.
 *
 * \retval 0 if successful
 * \retval -1 if failed
 */

int munmap(void *addr, size_t len)
{
	((void)len);

	return UnmapViewOfFile(addr) ? 0 : (errno = EINVAL, -1);
}

/** Writes any dirty pages within the given range to disk
 */
int msync(void *addr, size_t len, int flags)
{
	((void)flags);

	return FlushViewOfFile(addr, len) ? 0 : (errno = EINVAL, -1);
}


/* ////////////////////////////////////////////////////////////////////////// */
